#
# Copyright (C) 2022 Sebastiano Barezzi
#
# SPDX-License-Identifier: Apache-2.0
#

import requests
from requests.models import Response

class GoFileRequests:
	@classmethod
	def delete(cls, *args, **kwargs):
		response = cls._send_request(requests.delete, *args, **kwargs)
		return cls._process_response(response)

	@classmethod
	def get(cls, *args, **kwargs):
		response = cls._send_request(requests.get, *args, **kwargs)
		return cls._process_response(response)

	@classmethod
	def post(cls, *args, **kwargs):
		response = cls._send_request(requests.post, *args, **kwargs)
		return cls._process_response(response)

	@classmethod
	def put(cls, *args, **kwargs):
		response = cls._send_request(requests.put, *args, **kwargs)
		return cls._process_response(response)

	@staticmethod
	def _send_request(func, *args, **kwargs):
		for arg in ["data", "params"]:
			kwargs[arg] = kwargs.get(arg, {})
			# We do a little trolling
			kwargs[arg]["websiteToken"] = "websiteToken"

		response = func(*args, **kwargs)
		return response

	@staticmethod
	def _process_response(response: Response):
		response_json = response.json()
		if not "status" in response_json:
			raise Exception(f"Invalid response: {response_json}")

		if response_json["status"] != "ok":
			raise Exception(f"Error: {response_json['status']}")
		
		return response_json["data"]



#
# Copyright (C) 2022 Sebastiano Barezzi
#
# SPDX-License-Identifier: Apache-2.0
#
"""sorted()'s keys collection library."""

from functools import cmp_to_key
from locale import strcoll
from pathlib import Path
from typing import Union

from sebaubuntu_libs.libstring import removeprefix

def strcoll_files(string1: Union[str, Path], string2: Union[str, Path]) -> int:
	"""Reorder a file list by dir first, then name."""
	string1 = str(string1)
	string2 = str(string2)

	# Skip logic if one of the string if empty
	if not string1 or not string2:
		return strcoll(string1, string2)

	# If no directories, compare normally
	if not "/" in string1 and not "/" in string2:
		return strcoll(string1, string2)

	string1_dir = string1.rsplit("/", 1)[0] + "/"
	string2_dir = string2.rsplit("/", 1)[0] + "/"
	if string1_dir == string2_dir:
		# Same directory, compare normally
		return strcoll(string1, string2)

	if string1_dir.startswith(string2_dir):
		# First string dir is a subdirectory of the second one,
		# return string1 > string2
		return -1

	if string2_dir.startswith(string1_dir):
		# Second string dir is a subdirectory of the first one,
		# return string2 > string1
		return 1

	# Compare normally
	return strcoll(string1, string2)

strcoll_files_key = cmp_to_key(strcoll_files)

def strcoll_proprietary_files(string1: Union[str, Path], string2: Union[str, Path]):
	"""Reorder a LineageOS's extract-utils' proprietary files list."""
	string1 = str(string1)
	string2 = str(string2)

	# Remove '-' from strings if there,
	# it is used to indicate a build target
	string1 = removeprefix(string1, '-')
	string2 = removeprefix(string2, '-')

	return strcoll_files(string1, string2)

strcoll_proprietary_files_key = cmp_to_key(strcoll_proprietary_files)



#
# Copyright (C) 2022 Sebastiano Barezzi
#
# SPDX-License-Identifier: Apache-2.0
#

from datetime import datetime
from io import BytesIO
from sebaubuntu_libs.libgofile import DOMAIN
from sebaubuntu_libs.libgofile.raw_api.rest import GoFileRequests

__all__ = [
	'get_server',
	'upload_file',
	'get_content',
	'create_folder',
	'set_folder_option',
	'copy_content',
	'delete_content',
	'get_account_details',
]

API_URL = f"https://api.{DOMAIN}"

def get_server() -> str:
	"""Returns the best server available to receive files."""
	return GoFileRequests.get(f"{API_URL}/getServer")

def upload_file(server: str, file: BytesIO, token: str = None, folder_id: str = None,
                description: str = None, password: str = None, tags: str = None,
                expire: datetime = None):
	"""Upload one file on a specific server."""
	params = {}
	if token is not None:
		params["token"] = token
	if folder_id is not None:
		params["folderId"] = folder_id
	if description is not None:
		params["description"] = description
	if password is not None:
		params["password"] = password
	if tags is not None:
		params["tags"] = tags
	if expire is not None:
		params["expire"] = expire.timestamp()

	files = {"file": file}

	return GoFileRequests.post(f"{server}.{DOMAIN}/uploadFile", params=params, files=files)

def get_content(content_id: str, token: str):
	"""Get a specific content details."""
	params = {
		"contentId": content_id,
		"token": token,
	}

	return GoFileRequests.get(f"{API_URL}/getContent", params=params)

def create_folder(parent_folder_id: str, folder_name: str, token: str):
	"""Create a new folder."""
	data = {
		"parentFolderId": parent_folder_id,
		"folderName": folder_name,
		"token": token,
	}

	return GoFileRequests.put(f"{API_URL}/createFolder", data=data)

def set_folder_option(token: str, folder_id: str, option: str, value: str):
	"""Set an option on a folder."""
	data = {
		"token": token,
		"folderId": folder_id,
		"option": option,
		"value": value,
	}

	return GoFileRequests.put(f"{API_URL}/setFolderOption", data=data)

def copy_content(contents_id: str, folder_id_dest: str, token: str):
	"""Copy one or multiple contents to another folder."""
	data = {
		"contentsId": contents_id,
		"folderIdDest": folder_id_dest,
		"token": token,
	}

	return GoFileRequests.put(f"{API_URL}/copyContent", data=data)

def delete_content(contents_id: str, token: str):
	"""Delete one or multiple files/folders."""
	params = {
		"contentsId": contents_id,
		"token": token,
	}

	return GoFileRequests.delete(f"{API_URL}/deleteContent", params=params)

def get_account_details(token: str, all_details: bool = False):
	"""Get the account details."""
	params = {
		"token": token,
	}
	if all_details:
		params["allDetails"] = "true"

	return GoFileRequests.get(f"{API_URL}/getAccountDetails", params=params)

# Undocumented APIs

def get_geo():
    """Get the client geolocation."""
    return GoFileRequests.get(f"{API_URL}/getGeo")



#
# Copyright (C) 2022 Sebastiano Barezzi
#
# SPDX-License-Identifier: Apache-2.0
#

from sebaubuntu_libs.libgofile.utils import get_account, get_content

class Session:
	def __init__(self, token: str):
		self.token = token

		self.account = get_account(token)

	def get_root_content(self):
		return get_content(self.account.root_folder, self.token)

	def get_content(self, content_id: str):
		return get_content(content_id, self.token)



#
# Copyright (C) 2022 Sebastiano Barezzi
#
# SPDX-License-Identifier: Apache-2.0
#

from pathlib import Path
from typing import TYPE_CHECKING

from sebaubuntu_libs.libandroid.elf.elf import ELF

if TYPE_CHECKING:
	from sebaubuntu_libs.libandroid.partitions.partition import AndroidPartition
else:
	AndroidPartition = None

class SharedLibrary:
	def __init__(self,
	             name: str,
	             partition: AndroidPartition,
	            ):
		self.name = name
		self.partition = partition

		self.filename = f"{self.name}.so"

		path_32 = self.partition.path / "lib" / self.filename
		self.lib_32 = ELF(path_32) if path_32.is_file() else None

		path_64 = self.partition.path / "lib64" / self.filename
		self.lib_64 = ELF(path_64) if path_64.is_file() else None

		assert self.lib_32 or self.lib_64, f"Library {self.name} not found in {self.partition.path}"

		if self.is_multilib():
			assert (self.lib_32.needed_libraries == self.lib_64.needed_libraries,
			        f"32-bit and 64-bit versions of {self.name} have different dependencies")

		self.needed_libraries = (self.lib_64.needed_libraries
		                         if self.lib_64
		                         else self.lib_32.needed_libraries)

	def is_multilib(self) -> bool:
		"""Check if this library is multilib (both 32 and 64bit)."""
		return self.lib_32 and self.lib_64

	def get_path(self) -> Path:
		"""Get either 64bit library path if exists, else 32bit one."""
		return self.lib_64.path if self.lib_64 else self.lib_32.path



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sebaubuntu_libs',
 'sebaubuntu_libs.libaik',
 'sebaubuntu_libs.libandroid',
 'sebaubuntu_libs.libandroid.elf',
 'sebaubuntu_libs.libandroid.fstab',
 'sebaubuntu_libs.libandroid.partitions',
 'sebaubuntu_libs.libandroid.props',
 'sebaubuntu_libs.libandroid.vintf',
 'sebaubuntu_libs.libexception',
 'sebaubuntu_libs.libgofile',
 'sebaubuntu_libs.libgofile.raw_api',
 'sebaubuntu_libs.liblocale',
 'sebaubuntu_libs.liblogging',
 'sebaubuntu_libs.libnekobin',
 'sebaubuntu_libs.libpath',
 'sebaubuntu_libs.libreorder',
 'sebaubuntu_libs.libsed',
 'sebaubuntu_libs.libstring',
 'sebaubuntu_libs.libtyping']

package_data = \
{'': ['*']}

install_requires = \
['GitPython>=3.1.27,<4.0.0',
 'pyelftools>=0.29,<0.30',
 'requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'sebaubuntu-libs',
    'version': '1.3.1',
    'description': "SebaUbuntu's shared libs",
    'long_description': '# sebaubuntu_libs\n\n[![PyPi version](https://img.shields.io/pypi/v/sebaubuntu_libs)](https://pypi.org/project/sebaubuntu_libs/)\n[![Codacy Badge](https://app.codacy.com/project/badge/Grade/383072c93d5b4fa293237d42360e2170)](https://www.codacy.com/gh/SebaUbuntu/sebaubuntu_libs/dashboard?utm_source=github.com&amp;utm_medium=referral&amp;utm_content=SebaUbuntu/sebaubuntu_libs&amp;utm_campaign=Badge_Grade)\n\nA collection of code shared between my projects\n\n## Installation\n\n```sh\npip3 install sebaubuntu_libs\n```\n\n# License\n\n```\n#\n# Copyright (C) 2022 Sebastiano Barezzi\n#\n# SPDX-License-Identifier: Apache-2.0\n#\n```\n',
    'author': 'Sebastiano Barezzi',
    'author_email': 'barezzisebastiano@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/sebaubuntu-python/sebaubuntu_libs',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



#
# Copyright (C) 2022 Sebastiano Barezzi
#
# SPDX-License-Identifier: Apache-2.0
#
"""Android library."""

from typing import List

class _AndroidVersion:
	__ALL: List["_AndroidVersion"] = []

	def __init__(self,
	             version_code: str,
	             version_name: str,
	             api_version: int,
	            ):
		self.version_code = version_code
		self.version_name = version_name
		self.api_version = api_version

		version_name_split = self.version_name.split()
		self.version_short = version_name_split[0][0]

		self.__ALL.append(self)

	@classmethod
	def from_version_code(cls, version_code: str):
		for version in cls.__ALL:
			if version.version_code == version_code:
				return version
		return None

	@classmethod
	def from_version_name(cls, version_name: str):
		for version in cls.__ALL:
			if version.version_name == version_name:
				return version
		return None

	@classmethod
	def from_api_version(cls, api_version: int):
		for version in cls.__ALL:
			if version.api_version == api_version:
				return version
		return None

	@classmethod
	def from_version_short(cls, version_short: str):
		for version in cls.__ALL:
			if version.version_short == version_short:
				return version
		return None

class AndroidVersion(_AndroidVersion):
	BASE = _AndroidVersion("1.0", "1.0", 1)
	BASE_1_1 = _AndroidVersion("1.1", "1.1", 2)
	CUPCAKE = _AndroidVersion("1.5", "Cupcake", 3)
	DONUT = _AndroidVersion("1.6", "Donut", 4)
	ECLAIR = _AndroidVersion("2.0", "Eclair", 5)
	ECLAIR_0_1 = _AndroidVersion("2.0.1", "Eclair incremental update", 6)
	ECLAIR_MR1 = _AndroidVersion("2.1", "Eclair MR1", 7)
	FROYO = _AndroidVersion("2.2", "Froyo", 8)
	GINGERBREAD = _AndroidVersion("2.3", "Gingerbread", 9)
	GINGERBREAD_MR1 = _AndroidVersion("2.3.3", "Gingerbread MR1", 10)
	HONEYCOMB = _AndroidVersion("3.0", "Honeycomb", 11)
	HONEYCOMB_MR1 = _AndroidVersion("3.1", "Honeycomb MR1", 12)
	HONEYCOMB_MR2 = _AndroidVersion("3.2", "Honeycomb MR2", 13)
	ICE_CREAM_SANDWICH = _AndroidVersion("4.0", "Ice Cream Sandwich", 14)
	ICE_CREAM_SANDWICH_MR1 = _AndroidVersion("4.0.3", "Ice Cream Sandwich MR1", 15)
	JELLY_BEAN = _AndroidVersion("4.1", "Jelly Bean", 16)
	JELLY_BEAN_MR1 = _AndroidVersion("4.2", "Jelly Bean MR1", 17)
	JELLY_BEAN_MR2 = _AndroidVersion("4.3", "Jelly Bean MR2", 18)
	KITKAT = _AndroidVersion("4.4", "KitKat", 19)
	KITKAT_WATCH = _AndroidVersion("4.4W", "KitKat Watch", 20)
	LOLLIPOP = _AndroidVersion("5.0", "Lollipop", 21)
	LOLLIPOP_MR1 = _AndroidVersion("5.1", "Lollipop MR1", 22)
	M = _AndroidVersion("6.0", "Marshmallow", 23)
	N = _AndroidVersion("7.0", "Nougat", 24)
	N_MR1 = _AndroidVersion("7.1", "Nougat MR1", 25)
	O = _AndroidVersion("8.0", "Oreo", 26)
	O_MR1 = _AndroidVersion("8.1", "Oreo MR1", 27)
	P = _AndroidVersion("9", "Pie", 28)
	Q = _AndroidVersion("10", "Q", 29)
	R = _AndroidVersion("11", "R", 30)
	S = _AndroidVersion("12", "S", 31)
	S_V2 = _AndroidVersion("13", "Sv2", 32)
	TIRAMISU = _AndroidVersion("14", "Tiramisu", 33)
